@@ -124,6 +124,45 @@ wxString Catalog::HeaderData::ToString(const wxString& line_delim)
     return hdr;
 }
 
+void Catalog::HeaderData::NormalizeHeaderOrder()
+{
+    // This is the order of header lines in a POT file
+    // generated by GNU Gettext's xgettext utility, or
+    // rearranged by the msgmerge utility.
+    const wxString canonicalOrder[] =
+    {
+        "Project-Id-Version",
+        "Report-Msgid-Bugs-To",
+        "POT-Creation-Date",
+        "PO-Revision-Date",
+        "Last-Translator",
+        "Language-Team",
+        "Language",
+        "MIME-Version",
+        "Content-Type",
+        "Content-Transfer-Encoding",
+        "Plural-Forms"
+    };
+
+    const wxArrayString orderArr(sizeof(canonicalOrder) / sizeof(*canonicalOrder),
+        canonicalOrder);
+
+    // Sort standard header lines to the beginning of the header, in their
+    // canonical order, and the rest after them, in their original order.
+    std::stable_sort(m_entries.begin(), m_entries.end(),
+        [&orderArr](const Entry& a, const Entry& b)
+        {
+            auto coalesce = [&orderArr](int x) -> int {
+                if (x == wxNOT_FOUND)
+                    return 1 + static_cast<int>(orderArr.GetCount());
+                else
+                    return x;
+            };
+
+            return coalesce(orderArr.Index(a.Key)) < coalesce(orderArr.Index(b.Key));
+        });
+}
+
 void Catalog::HeaderData::UpdateDict()
 {
     SetHeader("Project-Id-Version", Project);
@@ -215,6 +254,8 @@ void Catalog::HeaderData::UpdateDict()
         path.Printf("X-Poedit-SearchPathExcluded-%i", i);
         SetHeader(path, SearchPathsExcluded[i]);
     }
+
+    NormalizeHeaderOrder();
 }
 
 void Catalog::HeaderData::ParseDict()
