@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2016-2020 Vaclav Slavik
+ *  Copyright (C) 2016-2021 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -168,6 +168,10 @@ wxColour ColorScheme::DoGet(Color color, Mode mode)
         // Backgrounds:
 
         case Color::SidebarBackground:
+            #ifdef __WXOSX__
+            if (@available(macOS 11.0, *))
+                return mode == Dark ? sRGB(46, 47, 50) : sRGB(240, 240, 240); // same as EditingThickSeparator
+            #endif
             return mode == Dark ? sRGB(45, 42, 41) : "#edf0f4";
 
         case Color::EditingBackground:
