@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2016-2020 Vaclav Slavik
+ *  Copyright (C) 2016-2021 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -31,13 +31,41 @@
 
 #ifdef __WXOSX__
     #include <wx/nativewin.h>
-    #if !wxCHECK_VERSION(3,1,0)
-        #include "wx_backports/nativewin.h"
-    #endif
+#endif
+
+#ifdef __WXMSW__
+    #include <wx/commandlinkbutton.h>
 #endif
 
 #include <memory>
 
+#if defined(__WXOSX__)
+typedef wxNativeWindow ActionButtonBase;
+#elif defined(__WXGTK__)
+typedef wxButton ActionButtonBase;
+#elif defined(__WXMSW__)
+typedef wxCommandLinkButton ActionButtonBase;
+#endif
+
+/// Larger button generating wxEVT_MENU, e.g. for welcome screen
+class ActionButton : public ActionButtonBase
+{
+public:
+    ActionButton(wxWindow *parent, wxWindowID winid, const wxString& symbolicName, const wxString& label, const wxString& note);
+
+private:
+#ifndef __WXOSX__
+    void OnPressed(wxCommandEvent& e);
+#endif
+
+#ifdef __WXMSW__
+    bool MSWOnDraw(WXDRAWITEMSTRUCT* wxdis) override;
+
+    wxString m_title, m_note;
+    wxFont m_titleFont;
+#endif
+};
+
 
 #ifdef __WXOSX__
 
