@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2020 Vaclav Slavik
+ *  Copyright (C) 1999-2021 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -26,6 +26,8 @@
 #include <wx/wx.h>
 #include <wx/clipbrd.h>
 #include <wx/config.h>
+#include <wx/cpp.h>
+#include <wx/filedlg.h>
 #include <wx/fs_zip.h>
 #include <wx/image.h>
 #include <wx/cmdline.h>
@@ -56,8 +58,9 @@
 #include <winsparkle.h>
 #endif
 
-#ifdef __WXGTK3__
+#ifdef __WXGTK__
 #include <glib.h>
+#include <gtk/gtk.h>
 #endif
 
 #include <unicode/uclean.h>
@@ -70,9 +73,11 @@
 #include "colorscheme.h"
 #include "concurrency.h"
 #include "configuration.h"
+#include "crowdin_gui.h"
 #include "edapp.h"
 #include "edframe.h"
 #include "extractors/extractor_legacy.h"
+#include "filemonitor.h"
 #include "manager.h"
 #include "prefsdlg.h"
 #include "chooselang.h"
@@ -82,6 +87,7 @@
 #include "http_client.h"
 #include "icons.h"
 #include "version.h"
+#include "recent_files.h"
 #include "str_helpers.h"
 #include "tm/transmem.h"
 #include "utility.h"
@@ -89,15 +95,11 @@
 #include "errors.h"
 #include "language.h"
 #include "crowdin_client.h"
+#include "welcomescreen.h"
 
 #ifdef __WXOSX__
 struct PoeditApp::NativeMacAppData
 {
-    NSMenu *recentMenu = nullptr;
-    NSMenuItem *recentMenuItem = nullptr;
-    NSMenu *windowMenu = nullptr;
-    NSMenuItem *windowMenuItem = nullptr;
-    wxMenuBar *menuBar = nullptr;
 #ifdef USE_SPARKLE
     NSObject *sparkleDelegate = nullptr;
 #endif
@@ -273,7 +275,6 @@ PoeditApp::PoeditApp()
 {
 #ifdef __WXOSX__
     m_nativeMacAppData.reset(new NativeMacAppData);
-    wxMenuBar::SetAutoWindowMenu(false);
 #endif
 }
 
@@ -338,7 +339,9 @@ bool PoeditApp::OnInit()
     SetDllDirectory(L"");
 #endif
 
-#ifdef __WXGTK3__
+#ifdef __WXGTK__
+    gtk_window_set_default_icon_name("net.poedit.Poedit");
+    g_set_application_name("Poedit");
     // Wayland compatibility, see https://wiki.gnome.org/Projects/GnomeShell/ApplicationBased
     g_set_prgname("net.poedit.Poedit");
 #endif
@@ -454,18 +457,12 @@ bool PoeditApp::OnInit()
     SetupLanguage();
 
 #ifdef __WXOSX__
-    wxMenuBar *bar = wxXmlResource::Get()->LoadMenuBar("mainmenu_mac_global");
-    TweakOSXMenuBar(bar);
-    wxMenuBar::MacSetCommonMenuBar(bar);
+    CreateMenu(Menu::Global);
     // so that help menu is correctly merged with system-provided menu
     // (see http://sourceforge.net/tracker/index.php?func=detail&aid=1600747&group_id=9863&atid=309863)
     s_macHelpMenuTitleName = _("&Help");
 #endif
 
-#ifndef __WXOSX__
-    FileHistory().Load(*wxConfig::Get());
-#endif
-
 #ifdef __WXMSW__
     AssociateFileTypeIfNeeded();
 #endif
@@ -513,13 +510,19 @@ bool PoeditApp::OnInit()
     // attempted to open MO files), shut the app down. Don't do this on macOS
     // where a) the initialization is finished after OnInit() and b) apps
     // without windows are OK.
-    if (!PoeditFrame::HasAnyWindow())
+    if (!PoeditFrame::HasAnyWindow() && !WelcomeWindow::GetIfActive())
         return false;
 #endif
 
     return true;
 }
 
+void PoeditApp::OnEventLoopEnter(wxEventLoopBase *loop)
+{
+    wxApp::OnEventLoopEnter(loop);
+    FileMonitor::EventLoopStarted();
+}
+
 int PoeditApp::OnExit()
 {
 #ifndef __WXOSX__
@@ -527,15 +530,18 @@ int PoeditApp::OnExit()
     m_remoteServer.reset();
 #endif
 
+#ifdef __WXMSW__
     // Keep any clipboard data available on Windows after the app terminates:
     wxTheClipboard->Flush();
+#endif
 
     // Make sure PoeditFrame instances schedules for deletion are deleted
     // early -- e.g. before wxConfig is destroyed, so they can save changes
     DeletePendingObjects();
 
+    FileMonitor::CleanUp();
     ColorScheme::CleanUp();
-
+    RecentFiles::CleanUp();
     TranslationMemory::CleanUp();
 
 #ifdef HAVE_HTTP_CLIENT
@@ -632,17 +638,11 @@ wxLayoutDirection PoeditApp::GetLayoutDirection() const
 
 void PoeditApp::OpenNewFile()
 {
-    PoeditFrame *unused = PoeditFrame::UnusedWindow(/*active=*/false);
-    if (unused)
-        unused->Raise();
-    else
-        PoeditFrame::CreateWelcome();
+    WelcomeWindow::GetAndActivate();
 }
 
 void PoeditApp::OpenFiles(const wxArrayString& names, int lineno)
 {
-    PoeditFrame *active = PoeditFrame::UnusedActiveWindow();
-
     for ( auto name: names )
     {
         // MO files cannot be opened directly in Poedit (yet), but they are
@@ -660,16 +660,8 @@ void PoeditApp::OpenFiles(const wxArrayString& names, int lineno)
             continue;
         }
 
-        if (active)
-        {
-            active->OpenFile(name, lineno);
-            active->Raise();
-            active = nullptr;
-        }
-        else
-        {
-            PoeditFrame::Create(name, lineno);
-        }
+        WelcomeWindow::HideActive();
+        PoeditFrame::Create(name, lineno);
     }
 }
 
@@ -712,7 +704,7 @@ void PoeditApp::OnInitCmdLine(wxCmdLineParser& parser)
                      _("handle a poedit:// URI"), wxCMD_LINE_VAL_STRING);
     parser.AddLongOption(CL_LINE,
                      _("go to item at given line number"), wxCMD_LINE_VAL_NUMBER);
-    parser.AddParam("catalog.po", wxCMD_LINE_VAL_STRING,
+    parser.AddParam("translation.po", wxCMD_LINE_VAL_STRING,
                     wxCMD_LINE_PARAM_OPTIONAL | wxCMD_LINE_PARAM_MULTIPLE);
 }
 
@@ -851,18 +843,15 @@ bool PoeditApp::OnExceptionInMainLoop()
 // ---------------------------------------------------------------------------
 
 BEGIN_EVENT_TABLE(PoeditApp, wxApp)
-#ifndef __WXMSW__
-   EVT_MENU           (wxID_NEW,                  PoeditApp::OnNew)
-   EVT_MENU           (XRCID("menu_new_from_pot"),PoeditApp::OnNew)
+   EVT_MENU           (wxID_NEW,                  PoeditApp::OnNewFromScratch)
+   EVT_MENU           (XRCID("menu_new_from_pot"),PoeditApp::OnNewFromPOT)
    EVT_MENU           (wxID_OPEN,                 PoeditApp::OnOpen)
  #ifdef HAVE_HTTP_CLIENT
    EVT_MENU           (XRCID("menu_open_crowdin"),PoeditApp::OnOpenFromCrowdin)
  #endif
- #ifndef __WXOSX__
-   EVT_MENU_RANGE     (wxID_FILE1, wxID_FILE9,    PoeditApp::OnOpenHist)
- #endif
-#endif // !__WXMSW__
+   EVT_COMMAND        (wxID_ANY, EVT_OPEN_RECENT_FILE, PoeditApp::OnOpenHist)
    EVT_MENU           (wxID_ABOUT,                PoeditApp::OnAbout)
+   EVT_MENU           (XRCID("menu_welcome"),     PoeditApp::OnWelcomeWindow)
    EVT_MENU           (XRCID("menu_manager"),     PoeditApp::OnManager)
    EVT_MENU           (wxID_EXIT,                 PoeditApp::OnQuit)
    EVT_MENU           (wxID_PREFERENCES,          PoeditApp::OnPreferences)
@@ -877,89 +866,204 @@ BEGIN_EVENT_TABLE(PoeditApp, wxApp)
 #endif
 END_EVENT_TABLE()
 
+namespace
+{
 
-// macOS and GNOME apps should open new documents in a new window. On Windows,
-// however, the usual thing to do is to open the new document in the already
-// open window and replace the current document.
-#ifndef __WXMSW__
+/// Information about the window invoking an event. Handles the difference between
+/// Windows (where opening a new file is done in the same file as the existing one,
+/// and so the action must not destroy data) and elsewhere (where new window is created).
+struct InvokingWindowProxy
+{
+    InvokingWindowProxy(const wxCommandEvent& e) : m_actionTarget(nullptr)
+    {
+        auto obj = e.GetEventObject();
+        wxWindow* win = nullptr;
+        auto menu = dynamic_cast<wxMenu*>(obj);
+        if (menu)
+            win = menu->GetWindow();
+        else
+            win = dynamic_cast<wxWindow*>(obj);
 
-#define TRY_FORWARD_TO_ACTIVE_WINDOW(funcCall)                          \
-    {                                                                   \
-        PoeditFrame *active = PoeditFrame::UnusedActiveWindow();        \
-        if ( active )                                                   \
-        {                                                               \
-            active->funcCall;                                           \
-            return;                                                     \
-        }                                                               \
+        if (win)
+            win = wxGetTopLevelParent(win);
+
+        m_shouldReactivateWelcomeWindow = false;
+        m_isFromWelcomeWindow = dynamic_cast<WelcomeWindow*>(win) != nullptr;
+#ifdef __WXOSX__
+        // we can't detect the window from global menu, so always assume welcome window must be hidden:
+        if (!win && menu)
+            m_isFromWelcomeWindow = true;
+#endif
+
+        m_invokingWindow = win;
+#ifdef __WXMSW__
+        m_actionTarget = dynamic_cast<PoeditFrame*>(win);
+#endif
     }
 
-void PoeditApp::OnNew(wxCommandEvent& event)
-{
-    TRY_FORWARD_TO_ACTIVE_WINDOW( OnNew(event) );
+    bool IsPerformingActionAllowed()
+    {
+#ifdef __WXMSW__
+        if (m_actionTarget)
+            return m_actionTarget->AskIfCanDiscardCurrentDoc();
+        else
+#endif
+            return true;
+    }
 
-    PoeditFrame *f = PoeditFrame::CreateEmpty();
-    f->OnNew(event);
+    void NotifyIsStarting()
+    {
+        if (m_isFromWelcomeWindow)
+            m_shouldReactivateWelcomeWindow = WelcomeWindow::HideActive();
+    }
+
+    void NotifyWasAborted() const
+    {
+        // restore welcome window if that's where the aborted action came from
+        if (m_shouldReactivateWelcomeWindow)
+            WelcomeWindow::GetAndActivate();
+    }
+
+    /// Window to perform actions (e.g. open files) in, or nullptr for new one
+    PoeditFrame *GetActionTarget() const
+    {
+        return m_actionTarget ? m_actionTarget : PoeditFrame::CreateEmpty();
+    }
+
+    /// Gets PoeditFrame that the action was invoken from; useful for e.g. file open window's parent
+    PoeditFrame *GetParentWindowIfAny() const { return m_actionTarget; }
+
+    /// Gets any invoking window; useful e.g. for parent of Upgrade-to-Pro prompt
+    wxWindow *GetInvokingWindow() const { return m_invokingWindow; }
+
+private:
+    bool m_shouldReactivateWelcomeWindow;
+    bool m_isFromWelcomeWindow;
+    wxWindow *m_invokingWindow;
+    PoeditFrame *m_actionTarget;
+};
+
+} // anonymous namespace
+
+
+void PoeditApp::OnNewFromScratch(wxCommandEvent& event)
+{
+    InvokingWindowProxy win(event);
+
+    if (!win.IsPerformingActionAllowed())
+        return;
+
+    win.NotifyIsStarting();
+    win.GetActionTarget()->NewFromScratch();
 }
 
 
-void PoeditApp::OnOpen(wxCommandEvent&)
+void PoeditApp::OnNewFromPOT(wxCommandEvent& event)
 {
-    PoeditFrame *active = PoeditFrame::UnusedActiveWindow();
+    InvokingWindowProxy win(event);
 
+    if (!win.IsPerformingActionAllowed())
+        return;
+
+    win.NotifyIsStarting();
     wxString path = wxConfig::Get()->Read("last_file_path", wxEmptyString);
 
-    wxFileDialog dlg(nullptr,
-                     MACOS_OR_OTHER("", _("Open catalog")),
+    wxFileDialog dlg(win.GetParentWindowIfAny(),
+        MACOS_OR_OTHER("", _("Select translation template")),
+        path,
+        wxEmptyString,
+        Catalog::GetTypesFileMask({ Catalog::Type::POT, Catalog::Type::PO }),
+        wxFD_OPEN | wxFD_FILE_MUST_EXIST);
+
+    if (dlg.ShowModal() != wxID_OK)
+    {
+        win.NotifyWasAborted();
+        return;
+    }
+
+    wxConfig::Get()->Write("last_file_path", dlg.GetDirectory());
+
+    auto pot = std::make_shared<POCatalog>(dlg.GetPath(), Catalog::CreationFlag_IgnoreTranslations);
+    if (!pot->IsOk())
+    {
+        win.NotifyWasAborted();
+        wxLogError(_(L"“%s” is not a valid POT file."), dlg.GetPath());
+        return;
+    }
+
+    // Silently fix duplicates because they are common in WP world:
+    if (pot->HasDuplicateItems())
+        pot->FixDuplicateItems();
+
+    win.GetActionTarget()->NewFromPOT(pot);
+}
+
+
+void PoeditApp::OnOpen(wxCommandEvent& event)
+{
+    InvokingWindowProxy win(event);
+
+    if (!win.IsPerformingActionAllowed())
+        return;
+
+    win.NotifyIsStarting();
+
+    wxString path = wxConfig::Get()->Read("last_file_path", wxEmptyString);
+    wxFileDialog dlg(win.GetParentWindowIfAny(),
+                     MACOS_OR_OTHER("", _("Select translation file")),
                      path,
                      wxEmptyString,
                      Catalog::GetAllTypesFileMask(),
                      wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_MULTIPLE);
 
-    if (dlg.ShowModal() == wxID_OK)
+    if (dlg.ShowModal() != wxID_OK)
     {
-        wxConfig::Get()->Write("last_file_path", dlg.GetDirectory());
-        wxArrayString paths;
-        dlg.GetPaths(paths);
-
-        if (paths.size() == 1 && active)
-        {
-            active->OpenFile(paths[0]);
-            return;
-        }
-
-        OpenFiles(paths);
+        win.NotifyWasAborted();
+        return;
     }
+
+    wxConfig::Get()->Write("last_file_path", dlg.GetDirectory());
+    wxArrayString paths;
+    dlg.GetPaths(paths);
+
+    win.GetActionTarget()->DoOpenFile(paths.front());
+    paths.erase(paths.begin());
+
+    if (!paths.empty())
+        OpenFiles(paths);
 }
 
 
 #ifdef HAVE_HTTP_CLIENT
 void PoeditApp::OnOpenFromCrowdin(wxCommandEvent& event)
 {
-    TRY_FORWARD_TO_ACTIVE_WINDOW( OnOpenFromCrowdin(event) );
+    InvokingWindowProxy win(event);
 
-    PoeditFrame *f = PoeditFrame::CreateEmpty();
-    f->OnOpenFromCrowdin(event);
+    if (!win.IsPerformingActionAllowed())
+        return;
+
+    win.NotifyIsStarting();
+    CrowdinOpenFile(win.GetParentWindowIfAny(), [=](int retval, wxString filename)
+    {
+        if (retval == wxID_OK)
+            win.GetActionTarget()->NewFromCrowdin(filename);
+        else
+            win.NotifyWasAborted();
+    });
 }
 #endif
 
 
-#ifndef __WXOSX__
 void PoeditApp::OnOpenHist(wxCommandEvent& event)
 {
-    TRY_FORWARD_TO_ACTIVE_WINDOW( OnOpenHist(event) );
+    InvokingWindowProxy win(event);
 
-    wxString f(FileHistory().GetHistoryFile(event.GetId() - wxID_FILE1));
-    if ( !wxFileExists(f) )
-    {
-        wxLogError(_(L"File “%s” doesn’t exist."), f.c_str());
+    if (!win.IsPerformingActionAllowed())
         return;
-    }
 
-    OpenFiles(wxArrayString(1, &f));
+    win.NotifyIsStarting();
+    win.GetActionTarget()->DoOpenFile(event.GetString());
 }
-#endif // !__WXOSX__
-
-#endif // !__WXMSW__
 
 
 void PoeditApp::OnAbout(wxCommandEvent&)
@@ -971,7 +1075,7 @@ void PoeditApp::OnAbout(wxCommandEvent&)
     about.SetVersion(wxGetApp().GetAppVersion());
     about.SetDescription(_("Poedit is an easy to use translation editor."));
 #endif
-    about.SetCopyright(L"Copyright \u00a9 1999-2020 Václav Slavík");
+    about.SetCopyright(L"Copyright \u00a9 1999-2021 Václav Slavík");
 #ifdef __WXGTK__ // other ports would show non-native about dlg
     about.SetWebSite("https://poedit.net");
 #endif
@@ -980,6 +1084,12 @@ void PoeditApp::OnAbout(wxCommandEvent&)
 }
 
 
+void PoeditApp::OnWelcomeWindow(wxCommandEvent&)
+{
+    WelcomeWindow::GetAndActivate();
+}
+
+
 void PoeditApp::OnManager(wxCommandEvent&)
 {
     wxFrame *f = ManagerFrame::Create();
@@ -1071,211 +1181,16 @@ void PoeditApp::MacOpenFiles(const wxArrayString& names)
     gs_lineToOpen = 0;
 }
 
-
-static NSMenuItem *AddNativeItem(NSMenu *menu, int pos, const wxString& text, SEL ac, NSString *key)
-{
-    NSString *str = str::to_NS(text);
-    if (pos == -1)
-        return [menu addItemWithTitle:str action:ac keyEquivalent:key];
-    else
-        return [menu insertItemWithTitle:str action:ac keyEquivalent:key atIndex:pos];
-}
-
-void PoeditApp::TweakOSXMenuBar(wxMenuBar *bar)
-{
-    wxMenu *apple = bar->OSXGetAppleMenu();
-    if (!apple)
-        return; // huh
-
-    apple->Insert(3, XRCID("menu_manager"), _("Catalogs Manager"));
-    apple->InsertSeparator(3);
-
-#if USE_SPARKLE
-    Sparkle_AddMenuItem(apple->GetHMenu(), _(L"Check for Updates…").utf8_str());
-#endif
-
-    wxMenu *fileMenu = nullptr;
-    wxMenuItem *fileCloseItem = bar->FindItem(wxID_CLOSE, &fileMenu);
-    if (fileMenu && fileCloseItem)
-    {
-        NSMenuItem *nativeCloseItem = [fileMenu->GetHMenu() itemWithTitle:str::to_NS(fileCloseItem->GetItemLabelText())];
-        if (nativeCloseItem)
-        {
-            nativeCloseItem.target = nil;
-            nativeCloseItem.action = @selector(performClose:);
-        }
-    }
-
-    int editMenuPos = bar->FindMenu(_("&Edit"));
-    if (editMenuPos == wxNOT_FOUND)
-        editMenuPos = 1;
-    wxMenu *edit = bar->GetMenu(editMenuPos);
-    int pasteItem = -1;
-    int findItem = -1;
-    int pos = 0;
-    for (auto& i : edit->GetMenuItems())
-    {
-        if (i->GetId() == wxID_PASTE)
-            pasteItem = pos;
-        else if (i->GetId() == XRCID("menu_sub_find"))
-            findItem = pos;
-        pos++;
-    }
-
-    NSMenu *editNS = edit->GetHMenu();
-
-    #pragma clang diagnostic push
-    #pragma clang diagnostic ignored "-Wundeclared-selector"
-    AddNativeItem(editNS, 0, _("Undo"), @selector(undo:), @"z");
-    AddNativeItem(editNS, 1, _("Redo"), @selector(redo:), @"Z");
-    #pragma clang diagnostic pop
-    [editNS insertItem:[NSMenuItem separatorItem] atIndex:2];
-    if (pasteItem != -1) pasteItem += 3;
-    if (findItem != -1) findItem += 3;
-
-    NSMenuItem *item;
-    if (pasteItem != -1)
-    {
-        item = AddNativeItem(editNS, pasteItem+1, _("Paste and Match Style"),
-                             @selector(pasteAsPlainText:), @"V");
-        [item setKeyEquivalentModifierMask:NSCommandKeyMask | NSAlternateKeyMask];
-        item = AddNativeItem(editNS, pasteItem+2, _("Delete"),
-                             @selector(delete:), @"");
-        [item setKeyEquivalentModifierMask:NSCommandKeyMask];
-        if (findItem != -1) findItem += 2;
-    }
-
-    #define FIND_PLUS(ofset) ((findItem != -1) ? (findItem+ofset) : -1)
-    if (findItem == -1)
-        [editNS addItem:[NSMenuItem separatorItem]];
-    item = AddNativeItem(editNS, FIND_PLUS(1), _("Spelling and Grammar"), NULL, @"");
-    NSMenu *spelling = [[NSMenu alloc] initWithTitle:@"Spelling and Grammar"];
-    AddNativeItem(spelling, -1, _("Show Spelling and Grammar"), @selector(showGuessPanel:), @":");
-    AddNativeItem(spelling, -1, _("Check Document Now"), @selector(checkSpelling:), @";");
-    [spelling addItem:[NSMenuItem separatorItem]];
-    AddNativeItem(spelling, -1, _("Check Spelling While Typing"), @selector(toggleContinuousSpellChecking:), @"");
-    AddNativeItem(spelling, -1, _("Check Grammar With Spelling"), @selector(toggleGrammarChecking:), @"");
-    AddNativeItem(spelling, -1, _("Correct Spelling Automatically"), @selector(toggleAutomaticSpellingCorrection:), @"");
-    [editNS setSubmenu:spelling forItem:item];
-
-    item = AddNativeItem(editNS, FIND_PLUS(2), _("Substitutions"), NULL, @"");
-    NSMenu *subst = [[NSMenu alloc] initWithTitle:@"Substitutions"];
-    AddNativeItem(subst, -1, _("Show Substitutions"), @selector(orderFrontSubstitutionsPanel:), @"");
-    [subst addItem:[NSMenuItem separatorItem]];
-    AddNativeItem(subst, -1, _("Smart Copy/Paste"), @selector(toggleSmartInsertDelete:), @"");
-    AddNativeItem(subst, -1, _("Smart Quotes"), @selector(toggleAutomaticQuoteSubstitution:), @"");
-    AddNativeItem(subst, -1, _("Smart Dashes"), @selector(toggleAutomaticDashSubstitution:), @"");
-    AddNativeItem(subst, -1, _("Smart Links"), @selector(toggleAutomaticLinkDetection:), @"");
-    AddNativeItem(subst, -1, _("Text Replacement"), @selector(toggleAutomaticTextReplacement:), @"");
-    [editNS setSubmenu:subst forItem:item];
-
-    item = AddNativeItem(editNS, FIND_PLUS(3), _("Transformations"), NULL, @"");
-    NSMenu *trans = [[NSMenu alloc] initWithTitle:@"Transformations"];
-    AddNativeItem(trans, -1, _("Make Upper Case"), @selector(uppercaseWord:), @"");
-    AddNativeItem(trans, -1, _("Make Lower Case"), @selector(lowercaseWord:), @"");
-    AddNativeItem(trans, -1, _("Capitalize"), @selector(capitalizeWord:), @"");
-    [editNS setSubmenu:trans forItem:item];
-
-    item = AddNativeItem(editNS, FIND_PLUS(4), _("Speech"), NULL, @"");
-    NSMenu *speech = [[NSMenu alloc] initWithTitle:@"Speech"];
-    AddNativeItem(speech, -1, _("Start Speaking"), @selector(startSpeaking:), @"");
-    AddNativeItem(speech, -1, _("Stop Speaking"), @selector(stopSpeaking:), @"");
-    [editNS setSubmenu:speech forItem:item];
-
-    int viewMenuPos = bar->FindMenu(_("&View"));
-    if (viewMenuPos != wxNOT_FOUND)
-    {
-        NSMenu *viewNS = bar->GetMenu(viewMenuPos)->GetHMenu();
-        [viewNS addItem:[NSMenuItem separatorItem]];
-        item = AddNativeItem(viewNS, -1, _("Enter Full Screen"), @selector(toggleFullScreen:), @"f");
-        [item setKeyEquivalentModifierMask:NSCommandKeyMask | NSControlKeyMask];
-
-    }
-
-    if (!m_nativeMacAppData->windowMenu)
-    {
-        NSMenu *windowMenu = [[NSMenu alloc] initWithTitle:str::to_NS(_("Window"))];
-        AddNativeItem(windowMenu, -1, _("Minimize"), @selector(performMiniaturize:), @"m");
-        AddNativeItem(windowMenu, -1, _("Zoom"), @selector(performZoom:), @"");
-        [windowMenu addItem:[NSMenuItem separatorItem]];
-        AddNativeItem(windowMenu, -1, _("Bring All to Front"), @selector(arrangeInFront:), @"");
-        [NSApp setWindowsMenu:windowMenu];
-        m_nativeMacAppData->windowMenu = windowMenu;
-    }
-}
-
-void PoeditApp::CreateFakeOpenRecentMenu()
-{
-    // Populate the menu with a hack that will be replaced.
-    NSMenu *mainMenu = [NSApp mainMenu];
- 
-    NSMenuItem *item = [mainMenu addItemWithTitle:@"File" action:NULL keyEquivalent:@""];
-    NSMenu *menu = [[NSMenu alloc] initWithTitle:NSLocalizedString(@"File", nil)];
- 
-    item = [menu addItemWithTitle:NSLocalizedString(@"Open Recent", nil)
-            action:NULL
-            keyEquivalent:@""];
-    NSMenu *openRecentMenu = [[NSMenu alloc] initWithTitle:@"Open Recent"];
-    #pragma clang diagnostic push
-    #pragma clang diagnostic ignored "-Wundeclared-selector"
-    [openRecentMenu performSelector:@selector(_setMenuName:) withObject:@"NSRecentDocumentsMenu"];
-    #pragma clang diagnostic pop
-    [menu setSubmenu:openRecentMenu forItem:item];
-    m_nativeMacAppData->recentMenuItem = item;
-    m_nativeMacAppData->recentMenu = openRecentMenu;
- 
-    item = [openRecentMenu addItemWithTitle:NSLocalizedString(@"Clear Menu", nil)
-            action:@selector(clearRecentDocuments:)
-            keyEquivalent:@""];
-}
-
-void PoeditApp::FixupMenusForMac(wxMenuBar *bar)
-{
-    m_nativeMacAppData->menuBar = nullptr;
-
-    if (m_nativeMacAppData->recentMenuItem)
-        [m_nativeMacAppData->recentMenuItem setSubmenu:nil];
-    if (m_nativeMacAppData->windowMenuItem)
-        [m_nativeMacAppData->windowMenuItem setSubmenu:nil];
-
-    if (!bar)
-        return;
-
-    wxMenu *fileMenu;
-    wxMenuItem *item = bar->FindItem(XRCID("open_recent"), &fileMenu);
-    if (item)
-    {
-        NSMenu *native = fileMenu->GetHMenu();
-        NSMenuItem *nativeItem = [native itemWithTitle:str::to_NS(item->GetItemLabelText())];
-        if (nativeItem)
-        {
-            [nativeItem setSubmenu:m_nativeMacAppData->recentMenu];
-            m_nativeMacAppData->recentMenuItem = nativeItem;
-        }
-    }
-
-    NSMenuItem *windowItem = [[NSApp mainMenu] itemWithTitle:str::to_NS(_("Window"))];
-    if (windowItem)
-    {
-        [windowItem setSubmenu:m_nativeMacAppData->windowMenu];
-        m_nativeMacAppData->windowMenuItem = windowItem;
-    }
-
-    m_nativeMacAppData->menuBar = bar;
-}
-
 void PoeditApp::OnIdleFixupMenusForMac(wxIdleEvent& event)
 {
     event.Skip();
-    auto installed = wxMenuBar::MacGetInstalledMenuBar();
-    if (m_nativeMacAppData->menuBar != installed)
-        FixupMenusForMac(installed);
+    FixupMenusForMacIfNeeded();
 }
 
 void PoeditApp::OSXOnWillFinishLaunching()
 {
     wxApp::OSXOnWillFinishLaunching();
-    CreateFakeOpenRecentMenu();
+    RecentFiles::Get().MacCreateFakeOpenRecentMenu();
     // We already create the menu item, this would cause duplicates "thanks" to the weird
     // way wx's menubar works on macOS:
     [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"NSFullScreenMenuItemEverywhere"];
