@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2020 Vaclav Slavik
+ *  Copyright (C) 1999-2021 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -39,7 +39,7 @@ class TranslationTextCtrl;
 class SwitchButton;
 
 class WXDLLIMPEXP_FWD_CORE wxBoxSizer;
-class WXDLLIMPEXP_FWD_CORE wxNotebook;
+class WXDLLIMPEXP_FWD_CORE wxBookCtrlBase;
 class WXDLLIMPEXP_FWD_CORE wxStaticText;
 
 
@@ -111,11 +111,12 @@ public:
     SourceTextCtrl *Ctrl_Original() const { return m_textOrig; }
     SourceTextCtrl *Ctrl_OriginalPlural() const { return m_textOrigPlural; }
     TranslationTextCtrl *Ctrl_Translation() const { return m_textTrans; }
-    wxNotebook *Ctrl_PluralNotebook() const { return m_pluralNotebook; }
+    wxBookCtrlBase *Ctrl_PluralNotebook() const { return m_pluralNotebook; }
     TranslationTextCtrl *Ctrl_PluralTranslation(size_t index) const { return m_textTransPlural[index]; }
 
 private:
     void UpdateAuxiliaryInfo(CatalogItemPtr item);
+    void UpdateCharCounter(CatalogItemPtr item);
 
     void CreateEditControls(wxBoxSizer *sizer);
     void CreateTemplateControls(wxBoxSizer *sizer);
@@ -130,6 +131,7 @@ private:
 private:
     class TagLabel;
     class IssueLabel;
+    class CharCounter;
 
     PoeditListCtrl *m_associatedList;
 
@@ -142,7 +144,7 @@ private:
     std::vector<TranslationTextCtrl*> m_textTransPlural;
     TranslationTextCtrl *m_textTransSingularForm;
 
-    wxNotebook *m_pluralNotebook;
+    wxBookCtrlBase *m_pluralNotebook;
     wxStaticText *m_labelSingular, *m_labelPlural;
     wxStaticText *m_labelSource, *m_labelTrans;
 
@@ -151,6 +153,8 @@ private:
     TagLabel *m_tagPretranslated;
 
     IssueLabel *m_issueLine;
+
+    CharCounter *m_charCounter;
 };
 
 #endif // Poedit_editing_area_h
