@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2000-2020 Vaclav Slavik
+ *  Copyright (C) 2000-2021 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -31,6 +31,7 @@
 
 #include <wx/dir.h>
 #include <wx/filename.h>
+#include <wx/textfile.h>
 
 #include <algorithm>
 
@@ -106,38 +107,40 @@ int FindInDir(const wxString& basepath, const wxString& dirname, const PathsToMa
         return 0;
 
     bool cont;
-    wxString filename;
+    wxString iter;
     int found = 0;
     
-    cont = dir.GetFirst(&filename, wxEmptyString, wxDIR_FILES);
+    cont = dir.GetFirst(&iter, wxEmptyString, wxDIR_FILES);
     while (cont)
     {
-        const wxString f = (dirname == ".") ? filename : dirname + "/" + filename;
-        cont = dir.GetNext(&filename);
+        const wxString filename = iter;
+        const wxString fullpath = (dirname == ".") ? filename : dirname + "/" + filename;
+        cont = dir.GetNext(&iter);
 
-        if (excludedPaths.MatchesFile(f))
+        if (excludedPaths.MatchesFile(fullpath))
             continue;
 
-        CheckReadPermissions(basepath, f);
-        wxLogTrace("poedit.extractor", "  - %s", f);
-        output.push_back(f);
+        CheckReadPermissions(basepath, fullpath);
+        wxLogTrace("poedit.extractor", "  - %s", fullpath);
+        output.push_back(fullpath);
         found++;
     }
 
-    cont = dir.GetFirst(&filename, wxEmptyString, wxDIR_DIRS);
+    cont = dir.GetFirst(&iter, wxEmptyString, wxDIR_DIRS);
     while (cont)
     {
-        const wxString f = (dirname == ".") ? filename : dirname + "/" + filename;
-        cont = dir.GetNext(&filename);
+        const wxString filename = iter;
+        const wxString fullpath = (dirname == ".") ? filename : dirname + "/" + filename;
+        cont = dir.GetNext(&iter);
 
         if (IsVCSDir(filename))
             continue;
 
-        if (excludedPaths.MatchesFile(f))
+        if (excludedPaths.MatchesFile(fullpath))
             continue;
 
-        CheckReadPermissions(basepath, f);
-        found += FindInDir(basepath, f, excludedPaths, output);
+        CheckReadPermissions(basepath, fullpath);
+        found += FindInDir(basepath, fullpath, excludedPaths, output);
     }
 
     return found;
@@ -179,7 +182,7 @@ Extractor::FilesList Extractor::CollectAllFiles(const SourceCodeSpec& sources)
         }
         else
         {
-            throw ExtractionException(ExtractionError::NoSourcesFound);
+            throw ExtractionException(ExtractionError::NoSourcesFound, path);
         }
     }
 
@@ -204,7 +207,7 @@ wxString Extractor::ExtractWithAll(TempDirectory& tmpdir,
 
     std::vector<wxString> subPots;
 
-    for (auto ex: CreateAllExtractors())
+    for (auto ex: CreateAllExtractors(sourceSpec))
     {
         const auto ex_files = ex->FilterFiles(files);
         if (ex_files.empty())
@@ -324,11 +327,29 @@ wxString Extractor::ConcatCatalogs(TempDirectory& tmpdir, const std::vector<wxSt
 
     auto outfile = tmpdir.CreateFileName("concatenated.pot");
 
-    wxString list;
-    for (auto& f: files)
-        list += wxString::Format(" %s", QuoteCmdlineArg(f));
+    wxTextFile filelist;
+    filelist.Create(tmpdir.CreateFileName("gettext_filelist.txt"));
+    for (auto fn: files)
+    {
+#ifdef __WXMSW__
+        // Gettext tools can't handle Unicode filenames well (due to using
+        // char* arguments), so work around this by using the short names.
+        if (!fn.IsAscii())
+        {
+            fn = CliSafeFileName(fn);
+            fn.Replace("\\", "/");
+        }
+#endif
+        filelist.AddLine(fn);
+    }
+    filelist.Write(wxTextFileType_Unix, wxConvFile);
 
-    auto cmd = wxString::Format("msgcat --force-po -o %s %s", QuoteCmdlineArg(outfile), list);
+    auto cmd = wxString::Format
+               (
+                   "msgcat --force-po -o %s --files-from=%s",
+                   QuoteCmdlineArg(outfile),
+                   QuoteCmdlineArg(filelist.GetName())
+               );
     bool succ = ExecuteGettext(cmd);
 
     if (!succ)
@@ -342,16 +363,20 @@ wxString Extractor::ConcatCatalogs(TempDirectory& tmpdir, const std::vector<wxSt
 }
 
 
-Extractor::ExtractorsList Extractor::CreateAllExtractors()
+Extractor::ExtractorsList Extractor::CreateAllExtractors(const SourceCodeSpec& sources)
 {
     ExtractorsList all;
 
-    // User-defined "legacy" extractors take precedence over anything else,
-    // to allow customization of the behavior:
-    CreateAllLegacyExtractors(all);
+    // User-defined "legacy" extractors customizing the behavior:
+    CreateAllLegacyExtractors(all, sources);
 
     // Standard builtin extractors follow
-    CreateGettextExtractors(all);
+    CreateGettextExtractors(all, sources);
+
+    std::stable_sort(all.begin(), all.end(), [](const auto& a, const auto& b)
+    {
+        return a->GetPriority() < b->GetPriority();
+    });
 
     return all;
 }
