@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2017-2020 Vaclav Slavik
+ *  Copyright (C) 2017-2021 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -47,6 +47,8 @@ struct SourceCodeSpec
     wxArrayString Keywords;
     wxString Charset;
 
+    std::vector<std::pair<wxString, wxString>> TypeMapping;
+
     // additional keys from the headers
     std::map<wxString, wxString> XHeaders;
 };
@@ -61,10 +63,11 @@ enum class ExtractionError
 class ExtractionException : public std::runtime_error
 {
 public:
-    ExtractionException(ExtractionError error_)
-        : std::runtime_error("extraction error"), error(error_) {}
+    ExtractionException(ExtractionError error_, const wxString& file_ = wxString())
+        : std::runtime_error("extraction error"), error(error_), file(file_) {}
 
     ExtractionError error;
+    wxString file;
 };
 
 
@@ -87,7 +90,7 @@ public:
         order, i.e. subsequent extractors should only be used to process files
         not yet handled by previous extractors.
      */
-    static ExtractorsList CreateAllExtractors();
+    static ExtractorsList CreateAllExtractors(const SourceCodeSpec& sources);
 
     /**
         Collects all files from source code, possibly including files that
@@ -121,6 +124,20 @@ public:
     /// Returns extractor's symbolic name
     virtual wxString GetId() const = 0;
 
+    /// Priority value for GetPriority()
+    enum class Priority
+    {
+        Highest = 1,
+        High    = 10,
+        Default = 100
+    };
+
+    /// Returns priority of the extractor
+    Priority GetPriority() const { return m_priority; }
+
+    /// Sets extractor's priority
+    void SetPriority(Priority p) { m_priority = p; }
+
     /**
         Returns whether the file is recognized.
         
@@ -129,6 +146,11 @@ public:
       */
     virtual bool IsFileSupported(const wxString& file) const;
 
+    /// Add a known extension or wildcard to be used by default IsFileSupported
+    /// (called from ctors)
+    void RegisterExtension(const wxString& ext);
+    void RegisterWildcard(const wxString& wildcard);
+
     /**
         Extracts translations from given source files using all
         available extractors.
@@ -141,28 +163,24 @@ public:
                              const std::vector<wxString>& files) const = 0;
 
 protected:
-    Extractor() {}
+    Extractor() : m_priority(Priority::Default) {}
     virtual ~Extractor() {}
 
     /// Check if file is supported based on its extension
     bool HasKnownExtension(const wxString& file) const;
 
-    /// Add a known extension or wildcard to be used by default IsFileSupported
-    /// (called from ctors)
-    void RegisterExtension(const wxString& ext);
-    void RegisterWildcard(const wxString& wildcard);
-
     /// Concatenates catalogs using msgcat
     static wxString ConcatCatalogs(TempDirectory& tmpdir, const std::vector<wxString>& files);
 
 private:
+    Priority m_priority;
     std::set<wxString> m_extensions;
     std::vector<wxString> m_wildcards;
 
 protected:
     // private factories:
-    static void CreateAllLegacyExtractors(ExtractorsList& into);
-    static void CreateGettextExtractors(ExtractorsList& into);
+    static void CreateAllLegacyExtractors(ExtractorsList& into, const SourceCodeSpec& sources);
+    static void CreateGettextExtractors(ExtractorsList& into, const SourceCodeSpec& sources);
 };
 
 #endif // Poedit_extractor_h
