@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2000-2020 Vaclav Slavik
+ *  Copyright (C) 2000-2021 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -286,6 +286,10 @@ wxString LegacyExtractorSpec::BuildCommand(const std::vector<wxString>& files,
 LegacyExtractor::LegacyExtractor(const LegacyExtractorSpec& spec)
     : m_spec(spec)
 {
+    // User-defined "legacy" extractors take precedence over standard extractors
+    // to allow customization of the behavior:
+    SetPriority(Priority::High);
+
     m_id = "legacy_";
     for (auto c: spec.Name)
     {
@@ -336,7 +340,7 @@ wxString LegacyExtractor::Extract(TempDirectory& tmpdir,
 }
 
 
-void Extractor::CreateAllLegacyExtractors(Extractor::ExtractorsList& into)
+void Extractor::CreateAllLegacyExtractors(Extractor::ExtractorsList& into, const SourceCodeSpec& /*sources*/)
 {
     // Extractors must be created anew to pick up any changes in definitions
 
