@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2001-2020 Vaclav Slavik
+ *  Copyright (C) 2001-2021 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -24,6 +24,7 @@
  */
 
 #include <wx/accel.h>
+#include <wx/bookctrl.h>
 #include <wx/choice.h>
 #include <wx/collpane.h>
 #include <wx/config.h>
@@ -32,7 +33,6 @@
 #include <wx/stattext.h>
 #include <wx/textctrl.h>
 #include <wx/checkbox.h>
-#include <wx/notebook.h>
 
 #ifdef __WXOSX__
 #include <AppKit/AppKit.h>
