@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2014-2020 Vaclav Slavik
+ *  Copyright (C) 2014-2021 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -95,14 +95,18 @@ SidebarBlock::SidebarBlock(Sidebar *parent, const wxString& label, int flags)
         if (!(flags & NoUpperMargin))
         {
             m_sizer->Add(new SidebarSeparator(parent),
-                         wxSizerFlags().Expand().Border(wxBOTTOM|wxLEFT|wxRIGHT, PX(4)));
+                         wxSizerFlags().Expand().Border(wxBOTTOM|wxLEFT|wxRIGHT, PX(5)));
         }
         m_headerSizer = new wxBoxSizer(wxHORIZONTAL);
         m_headerSizer->Add(new HeadingLabel(parent, label), wxSizerFlags().Center());
-        m_sizer->Add(m_headerSizer, wxSizerFlags().Expand().PXDoubleBorder(wxLEFT|wxRIGHT));
+        m_sizer->Add(m_headerSizer, wxSizerFlags().Expand().Border(wxLEFT|wxRIGHT, SIDEBAR_PADDING));
     }
     m_innerSizer = new wxBoxSizer(wxVERTICAL);
-    m_sizer->Add(m_innerSizer, wxSizerFlags(1).Expand().PXDoubleBorder(wxLEFT|wxRIGHT));
+
+    auto innerFlags = wxSizerFlags(1).Expand();
+    if (!(flags & NoSideMargins))
+        innerFlags.Border(wxLEFT|wxRIGHT, SIDEBAR_PADDING);
+    m_sizer->Add(m_innerSizer, innerFlags);
 }
 
 void SidebarBlock::Show(bool show)
@@ -130,7 +134,7 @@ class OldMsgidSidebarBlock : public SidebarBlock
 public:
     OldMsgidSidebarBlock(Sidebar *parent)
           /// TRANSLATORS: "Previous" as in used in the past, now replaced with newer.
-        : SidebarBlock(parent, _("Previous source text:"))
+        : SidebarBlock(parent, _("Previous source text"))
     {
         m_innerSizer->AddSpacer(PX(2));
         m_innerSizer->Add(new ExplanationLabel(parent, _("The old source text (before it changed during an update) that the now-inaccurate translation corresponds to.")),
@@ -159,7 +163,7 @@ class ExtractedCommentSidebarBlock : public SidebarBlock
 {
 public:
     ExtractedCommentSidebarBlock(Sidebar *parent)
-        : SidebarBlock(parent, _("Notes for translators:"))
+        : SidebarBlock(parent, _("Notes for translators"))
     {
         m_innerSizer->AddSpacer(PX(5));
         m_comment = new SelectableAutoWrappingText(parent, "");
@@ -192,7 +196,7 @@ class CommentSidebarBlock : public SidebarBlock
 {
 public:
     CommentSidebarBlock(Sidebar *parent)
-        : SidebarBlock(parent, _("Comment:"))
+        : SidebarBlock(parent, _("Comment"))
     {
         m_innerSizer->AddSpacer(PX(5));
         m_comment = new SelectableAutoWrappingText(parent, "");
@@ -260,9 +264,9 @@ wxDEFINE_EVENT(EVT_SUGGESTION_SELECTED, wxCommandEvent);
 class SuggestionWidget : public wxWindow
 {
 public:
-    SuggestionWidget(Sidebar *parent, SuggestionsSidebarBlock *block, bool isFirst) : wxWindow(parent, wxID_ANY)
+    SuggestionWidget(Sidebar *sidebar, wxWindow *parent, SuggestionsSidebarBlock *block, bool isFirst) : wxWindow(parent, wxID_ANY)
     {
-        m_sidebar = parent;
+        m_sidebar = sidebar;
         m_parentBlock = block;
         m_isHighlighted = false;
         m_icon = new StaticBitmap(this, "SuggestionTMTemplate");
@@ -274,12 +278,34 @@ public:
                       ? new wxStaticBitmap(this, wxID_ANY, wxArtProvider::GetBitmap("SuggestionPerfectMatch"))
                       : nullptr;
 
+        // Calculate the correct DPI-dependent offset of m_icon vs m_text - we want the icon centered
+        // on the first line of text.
+        const int textPadding = PX(6);
+#if defined(__WXOSX__)
+        const int iconPadding = PX(7);
+#elif defined(__WXMSW__)
+        int iconPadding = 0;
+        const auto hidpiFactor = HiDPIScalingFactor();
+        if (hidpiFactor < 1.25)
+            iconPadding = PX(7);
+        else if (hidpiFactor < 1.5)
+            iconPadding = PX(9)+1;
+        else if (hidpiFactor < 1.75)
+            iconPadding = PX(8)+1;
+        else if (hidpiFactor < 2.0)
+            iconPadding = PX(10);
+        else
+            iconPadding = PX(8)+1;
+#else
+        const int iconPadding = PX(7);
+#endif
+
         auto top = new wxBoxSizer(wxHORIZONTAL);
         auto right = new wxBoxSizer(wxVERTICAL);
-        top->AddSpacer(PX(2));
-        top->Add(m_icon, wxSizerFlags().Top().Border(wxTOP|wxBOTTOM, PX(6)));
-        top->Add(right, wxSizerFlags(1).Expand().PXBorder(wxLEFT));
-        right->Add(m_text, wxSizerFlags().Expand().Border(wxTOP, PX(4)));
+        top->AddSpacer(PX(6));
+        top->Add(m_icon, wxSizerFlags().Top().Border(wxTOP, iconPadding));
+        top->Add(right, wxSizerFlags(1).Expand().Border(wxLEFT, PX(8)));
+        right->Add(m_text, wxSizerFlags().Expand().Border(wxTOP, textPadding));
         auto infoSizer = new wxBoxSizer(wxHORIZONTAL);
         infoSizer->Add(m_info, wxSizerFlags().Center());
         if (m_isPerfect)
@@ -295,15 +321,16 @@ public:
         ColorScheme::SetupWindowColors(this, [=]
         {
             // setup mouse hover highlighting:
-            m_bg = parent->GetBackgroundColour();
+            auto bg = parent->GetBackgroundColour();
+            SetBackgroundColour(bg);
+#ifndef __WXOSX__
+            m_bg = bg;
             m_bgHighlight = ColorScheme::GetWindowMode(parent) == ColorScheme::Dark
                             ? m_bg.ChangeLightness(110)
                             : m_bg.ChangeLightness(95);
-            SetBackgroundColour(m_isHighlighted ? m_bgHighlight : m_bg);
-            #ifndef __WXOSX__
             for (auto c: GetChildren())
                 c->SetBackgroundColour(m_isHighlighted ? m_bgHighlight : m_bg);
-            #endif
+#endif
         });
 
         wxWindow* parts [] = { this, m_icon, m_text, m_info, m_moreActions };
@@ -360,14 +387,16 @@ public:
 #endif
         (void)tooltip;
 
+#ifndef __WXOSX__
         SetBackgroundColour(m_bg);
+#endif
 
         Layout();
         InvalidateBestSize();
         SetMinSize(wxDefaultSize);
         SetMinSize(GetBestSize());
     }
-    
+
     bool AcceptsFocus() const override { return false; }
 
 private:
@@ -395,14 +424,33 @@ private:
         wxPaintDC dc(this);
         if (m_isHighlighted)
         {
+#ifdef __WXOSX__
+            auto winbg = GetBackgroundColour();
+            wxColour highlight;
+            if (@available(macOS 10.14, *))
+            {
+                NSColor *bg = winbg.OSXGetNSColor();
+                NSColor *osHighlight = [bg colorWithSystemEffect:NSColorSystemEffectRollover];
+                // use only lighter version of the highlight by blending with the background
+                highlight = wxColour([bg blendedColorWithFraction:0.2 ofColor:osHighlight]);
+            }
+            else
+            {
+                highlight = winbg.ChangeLightness(95);
+            }
+#else
+            auto highlight = m_bgHighlight;
+#endif
             std::unique_ptr<wxGraphicsContext> gc(wxGraphicsContext::Create(dc));
-            gc->SetBrush(m_bgHighlight);
+            gc->SetBrush(highlight);
             gc->SetPen(*wxTRANSPARENT_PEN);
 
             auto rect = GetClientRect();
             if (!rect.IsEmpty())
             {
-#if wxCHECK_VERSION(3,1,1)
+#if defined(__WXOSX__)
+                gc->DrawRoundedRectangle(rect.x, rect.y, rect.width, rect.height, PX(5));
+#elif wxCHECK_VERSION(3,1,1)
                 gc->DrawRoundedRectangle(rect.x, rect.y, rect.width, rect.height, PX(2));
 #else
                 gc->DrawRectangle(rect.x, rect.y, rect.width, rect.height);
@@ -442,18 +490,18 @@ private:
         auto suggestion = m_value;
         static const auto idDelete = wxNewId();
 
-        wxMenu *menu = new wxMenu();
+        wxMenu menu;
 #ifdef __WXOSX__
-        [menu->GetHMenu() setFont:[NSFont systemFontOfSize:13]];
+        [menu.GetHMenu() setFont:[NSFont systemFontOfSize:13]];
 #endif
-        menu->Append(idDelete, MSW_OR_OTHER(_("Delete from translation memory"), _("Delete From Translation Memory")));
-        menu->Bind(wxEVT_MENU, [sidebar,suggestion](wxCommandEvent&)
+        menu.Append(idDelete, MSW_OR_OTHER(_("Delete from translation memory"), _("Delete From Translation Memory")));
+        menu.Bind(wxEVT_MENU, [sidebar,suggestion](wxCommandEvent&)
         {
             SuggestionsProvider::Delete(suggestion);
             sidebar->RefreshContent();
         }, idDelete);
 
-        PopupMenu(menu);
+        PopupMenu(&menu);
     }
 
     void Highlight(bool highlight)
@@ -490,12 +538,20 @@ private:
     wxStaticText *m_info;
     wxStaticBitmap *m_isPerfect;
     ImageButton *m_moreActions;
+#ifndef __WXOSX__
     wxColour m_bg, m_bgHighlight;
+#endif
 };
 
 
 SuggestionsSidebarBlock::SuggestionsSidebarBlock(Sidebar *parent, wxMenu *menu)
-    : SidebarBlock(parent, _("Translation suggestions:"), NoUpperMargin),
+    : SidebarBlock(parent,
+                   // TRANSLATORS: as in: translation suggestions, suggested translations; should be similarly short
+                   _("Suggestions"),
+                   #if 0
+                   _("Translation suggestions"),
+                   #endif
+                   NoUpperMargin | NoSideMargins),
       m_suggestionsMenu(menu),
       m_msgPresent(false),
       m_suggestionsSeparator(nullptr),
@@ -504,22 +560,34 @@ SuggestionsSidebarBlock::SuggestionsSidebarBlock(Sidebar *parent, wxMenu *menu)
       m_lastUpdateTime(0)
 {
     m_provider.reset(new SuggestionsProvider);
+}
+
+void SuggestionsSidebarBlock::InitMainPanel()
+{
+    m_suggestionsPanel = new wxPanel(m_parent, wxID_ANY);
+    m_panelSizer = new wxBoxSizer(wxVERTICAL);
+    m_suggestionsPanel->SetSizer(m_panelSizer);
+
+    m_innerSizer->Add(m_suggestionsPanel, wxSizerFlags(1).Expand().Border(wxLEFT|wxRIGHT, SIDEBAR_PADDING));
+}
+
+void SuggestionsSidebarBlock::InitControls()
+{
+    InitMainPanel();
 
     m_msgSizer = new wxBoxSizer(wxHORIZONTAL);
-    m_msgIcon = new StaticBitmap(parent, wxString());
-    m_msgText = new ExplanationLabel(parent, "");
+    m_msgIcon = new StaticBitmap(m_suggestionsPanel, wxString());
+    m_msgText = new ExplanationLabel(m_suggestionsPanel, "");
     m_msgSizer->Add(m_msgIcon, wxSizerFlags().Center().PXBorderAll());
     m_msgSizer->Add(m_msgText, wxSizerFlags(1).Center().PXBorder(wxTOP|wxBOTTOM));
-    m_innerSizer->Add(m_msgSizer, wxSizerFlags().Expand());
-
-    m_innerSizer->AddSpacer(PX(10));
+    m_panelSizer->Add(m_msgSizer, wxSizerFlags().Expand().Border(wxBOTTOM, PX(10)));
 
     m_suggestionsSizer = new wxBoxSizer(wxVERTICAL);
     m_extrasSizer = new wxBoxSizer(wxVERTICAL);
-    m_innerSizer->Add(m_suggestionsSizer, wxSizerFlags().Expand());
-    m_innerSizer->Add(m_extrasSizer, wxSizerFlags().Expand());
+    m_panelSizer->Add(m_suggestionsSizer, wxSizerFlags().Expand());
+    m_panelSizer->Add(m_extrasSizer, wxSizerFlags().Expand());
 
-    m_iGotNothing = new wxStaticText(parent, wxID_ANY,
+    m_iGotNothing = new wxStaticText(m_suggestionsPanel, wxID_ANY,
                                 #ifdef __WXMSW__
                                      // TRANSLATORS: This is shown when no translation suggestions can be found in the TM (Windows).
                                      _("No matches found")
@@ -534,14 +602,15 @@ SuggestionsSidebarBlock::SuggestionsSidebarBlock(Sidebar *parent, wxMenu *menu)
 #endif
     ColorScheme::SetupWindowColors(m_iGotNothing, [=]
     {
+        m_suggestionsPanel->SetBackgroundColour(m_parent->GetBackgroundColour());
         m_iGotNothing->SetForegroundColour(ExplanationLabel::GetTextColor().ChangeLightness(150));
     });
-    m_innerSizer->Add(m_iGotNothing, wxSizerFlags().Center().Border(wxTOP|wxBOTTOM, PX(100)));
+    m_panelSizer->Add(m_iGotNothing, wxSizerFlags().Center().Border(wxTOP|wxBOTTOM, PX(100)));
 
     BuildSuggestionsMenu();
 
-    m_suggestionsTimer.SetOwner(parent);
-    parent->Bind(wxEVT_TIMER,
+    m_suggestionsTimer.SetOwner(m_parent);
+    m_parent->Bind(wxEVT_TIMER,
                  &SuggestionsSidebarBlock::OnDelayedShowSuggestionsForItem, this,
                  m_suggestionsTimer.GetId());
 }
@@ -568,7 +637,7 @@ void SuggestionsSidebarBlock::ClearMessage()
     m_msgPresent = false;
     m_msgText->SetAndWrapLabel("");
     UpdateVisibility();
-    m_parent->Layout();
+    m_suggestionsPanel->Layout();
 }
 
 void SuggestionsSidebarBlock::SetMessage(const wxString& icon, const wxString& text)
@@ -577,7 +646,7 @@ void SuggestionsSidebarBlock::SetMessage(const wxString& icon, const wxString& t
     m_msgIcon->SetBitmapName(icon);
     m_msgText->SetAndWrapLabel(text);
     UpdateVisibility();
-    m_parent->Layout();
+    m_suggestionsPanel->Layout();
 }
 
 void SuggestionsSidebarBlock::ReportError(SuggestionsBackend*, dispatch::exception_ptr e)
@@ -594,7 +663,7 @@ void SuggestionsSidebarBlock::ClearSuggestions()
 
 void SuggestionsSidebarBlock::UpdateSuggestions(const SuggestionsList& hits)
 {
-    wxWindowUpdateLocker lock(m_parent);
+    wxWindowUpdateLocker lock(m_suggestionsPanel);
 
     for (auto& h: hits)
     {
@@ -608,11 +677,11 @@ void SuggestionsSidebarBlock::UpdateSuggestions(const SuggestionsList& hits)
     // create any necessary controls:
     while (m_suggestions.size() > m_suggestionsWidgets.size())
     {
-        auto w = new SuggestionWidget(m_parent, this, /*isFirst=*/m_suggestionsWidgets.empty());
+        auto w = new SuggestionWidget(m_parent, m_suggestionsPanel, this, /*isFirst=*/m_suggestionsWidgets.empty());
         m_suggestionsSizer->Add(w, wxSizerFlags().Expand());
         m_suggestionsWidgets.push_back(w);
     }
-    m_innerSizer->Layout();
+    m_panelSizer->Layout();
 
     // update shown suggestions:
 
@@ -638,17 +707,17 @@ void SuggestionsSidebarBlock::UpdateSuggestions(const SuggestionsList& hits)
             if (perfectMatches > 1)
             {
                 if (!m_suggestionsSeparator)
-                    m_suggestionsSeparator = new SidebarSeparator(m_parent);
+                    m_suggestionsSeparator = new SidebarSeparator(m_suggestionsPanel);
                 m_suggestionsSeparator->Show();
-                m_suggestionsSizer->Insert(i, m_suggestionsSeparator, wxSizerFlags().Expand().Border(wxBOTTOM, MSW_OR_OTHER(PX(2), PX(4))));
+                m_suggestionsSizer->Insert(i, m_suggestionsSeparator, wxSizerFlags().Expand().Border(wxTOP|wxBOTTOM, MSW_OR_OTHER(PX(2), PX(4))));
             }
             perfectMatches = 0;
         }
     }
 
-    m_innerSizer->Layout();
+    m_panelSizer->Layout();
     UpdateVisibility();
-    m_parent->Layout();
+    m_suggestionsPanel->Layout();
 
     UpdateSuggestionsMenu();
 }
@@ -728,17 +797,17 @@ void SuggestionsSidebarBlock::OnQueriesFinished()
 {
     if (m_suggestions.empty())
     {
-        m_innerSizer->Show(m_iGotNothing);
-        m_parent->Layout();
+        m_panelSizer->Show(m_iGotNothing);
+        m_suggestionsPanel->Layout();
     }
 }
 
 void SuggestionsSidebarBlock::UpdateVisibility()
 {
     m_msgSizer->ShowItems(m_msgPresent);
-    m_innerSizer->Show(m_iGotNothing, m_suggestions.empty() && !m_pendingQueries);
+    m_panelSizer->Show(m_iGotNothing, m_suggestions.empty() && !m_pendingQueries);
 
-    int heightRemaining = m_innerSizer->GetSize().y;
+    int heightRemaining = m_panelSizer->GetSize().y;
     size_t w = 0;
     for (w = 0; w < m_suggestions.size(); w++)
     {
@@ -889,8 +958,8 @@ Sidebar::Sidebar(wxWindow *parent, wxMenu *suggestionsMenu)
     topSizer->SetMinSize(wxSize(PX(300), -1));
 
     m_blocksSizer = new wxBoxSizer(wxVERTICAL);
-    topSizer->Add(m_blocksSizer, wxSizerFlags(1).Expand().PXBorder(wxTOP|wxBOTTOM));
-    topSizer->AddSpacer(PXDefaultBorder);
+    topSizer->Add(m_blocksSizer, wxSizerFlags(1).Expand());
+    topSizer->AddSpacer(SIDEBAR_PADDING);
 
     m_topBlocksSizer = new wxBoxSizer(wxVERTICAL);
     m_bottomBlocksSizer = new wxBoxSizer(wxVERTICAL);
@@ -899,7 +968,8 @@ Sidebar::Sidebar(wxWindow *parent, wxMenu *suggestionsMenu)
     m_blocksSizer->Add(m_bottomBlocksSizer, wxSizerFlags().Expand());
 
     m_topBlocksSizer->AddSpacer(PXDefaultBorder);
-    AddBlock(new SuggestionsSidebarBlock(this, suggestionsMenu), Top);
+
+    AddBlock(SuggestionsSidebarBlock::Create(this, suggestionsMenu), Top);
     AddBlock(new OldMsgidSidebarBlock(this), Bottom);
     AddBlock(new ExtractedCommentSidebarBlock(this), Bottom);
     AddBlock(new CommentSidebarBlock(this), Bottom);
@@ -987,7 +1057,8 @@ void Sidebar::SetUpperHeight(int size)
         pos = pos / 2 - PX(1);
     }
 
-    pos += PX(5);
+    pos -= SIDEBAR_PADDING;
+    pos += PX(15) ; // SidebarSeparator spacing
 
     m_bottomBlocksSizer->SetMinSize(wxSize(-1, pos));
     Layout();
