@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2014-2020 Vaclav Slavik
+ *  Copyright (C) 2014-2021 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -51,6 +51,7 @@ class SuggestionsProvider;
 class SuggestionWidget;
 class Sidebar;
 
+#define SIDEBAR_PADDING PX(8)
 
 /// Implements part of the sidebar.
 class SidebarBlock : public std::enable_shared_from_this<SidebarBlock>
@@ -73,7 +74,8 @@ public:
 protected:
     enum Flags
     {
-        NoUpperMargin = 1
+        NoUpperMargin = 1,
+        NoSideMargins = 2
     };
 
     SidebarBlock(Sidebar *parent, const wxString& label, int flags = 0);
@@ -91,7 +93,13 @@ wxDECLARE_EVENT(EVT_SUGGESTION_SELECTED, wxCommandEvent);
 class SuggestionsSidebarBlock : public SidebarBlock
 {
 public:
-    SuggestionsSidebarBlock(Sidebar *parent, wxMenu *menu);
+    static SuggestionsSidebarBlock* Create(Sidebar *parent, wxMenu *menu)
+    {
+        auto s = new SuggestionsSidebarBlock(parent, menu);
+        s->InitControls();
+        return s;
+    }
+
     ~SuggestionsSidebarBlock();
 
     void Show(bool show) override;
@@ -100,6 +108,10 @@ public:
     void Update(const CatalogItemPtr& item) override;
 
 protected:
+    SuggestionsSidebarBlock(Sidebar *parent, wxMenu *menu);
+    virtual void InitControls();
+    virtual void InitMainPanel();
+
     // How many entries can have shortcuts?
     static const int SUGGESTIONS_MENU_ENTRIES = 9;
 
@@ -130,6 +142,8 @@ protected:
 protected:
     std::unique_ptr<SuggestionsProvider> m_provider;
 
+    wxWindow *m_suggestionsPanel;
+    wxSizer *m_panelSizer;
     wxMenu *m_suggestionsMenu;
 
     wxSizer *m_msgSizer;
