@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2014-2020 Vaclav Slavik
+ *  Copyright (C) 2014-2021 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -194,6 +194,13 @@ std::wregex RE_PHP_FORMAT(LR"(%(\d+\$)?[-+]{0,2}([ 0]|'.)?-?\d*(\..?\d+)?[%bcdeE
 std::wregex RE_C_FORMAT(LR"(%(\d+\$)?[-+ #0]{0,5}(\d+|\*)?(\.(\d+|\*))?(hh|ll|[hljztL])?[%csdioxXufFeEaAgGnp])",
                         std::regex_constants::ECMAScript | std::regex_constants::optimize);
 
+// python-format old style https://docs.python.org/2/library/stdtypes.html#string-formatting
+//               new style https://docs.python.org/3/library/string.html#format-string-syntax
+std::wregex RE_PYTHON_FORMAT(LR"((%(\(\w+\))?[-+ #0]?(\d+|\*)?(\.(\d+|\*))?[hlL]?[diouxXeEfFgGcrs%]))" // old style
+                             "|"
+                             LR"((\{([^{}])*\}))", // new style, being permissive
+                             std::regex_constants::ECMAScript | std::regex_constants::optimize);
+
 // ruby-format per https://ruby-doc.org/core-2.7.1/Kernel.html#method-i-sprintf
 std::wregex RE_RUBY_FORMAT(LR"(%(\d+\$)?[-+ #0]{0,5}(\d+|\*)?(\.(\d+|\*))?(hh|ll|[hljztL])?[%csdioxXufFeEaAgGnp])",
                            std::regex_constants::ECMAScript | std::regex_constants::optimize);
@@ -242,13 +249,18 @@ SyntaxHighlighterPtr SyntaxHighlighter::ForItem(const CatalogItem& item)
         static auto c_format = std::make_shared<RegexSyntaxHighlighter>(RE_C_FORMAT, TextKind::Format);
         all->Add(c_format);
     }
+    else if (formatFlag == "python")
+    {
+        static auto python_format = std::make_shared<RegexSyntaxHighlighter>(RE_PYTHON_FORMAT, TextKind::Format);
+        all->Add(python_format);
+    }
     else if (formatFlag == "ruby")
     {
         static auto ruby_format = std::make_shared<RegexSyntaxHighlighter>(RE_RUBY_FORMAT, TextKind::Format);
         all->Add(ruby_format);
     }
 
-    // basic higlighting has highest priority, so should come last in the order:
+    // basic highlighting has highest priority, so should come last in the order:
     all->Add(basic);
 
     return all;
