@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2013-2020 Vaclav Slavik
+ *  Copyright (C) 2013-2021 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -97,6 +97,8 @@ public:
      */
     void ImportData(std::function<void(IOInterface&)> source);
 
+    void SearchSubstring(IOInterface& destination,
+                         const Language& srclang, const Language& lang, const std::wstring& sourcePhrase);
 
     /**
         Performs updates to the translation memory.
