@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2013-2020 Vaclav Slavik
+ *  Copyright (C) 2013-2021 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -26,7 +26,18 @@
 #ifndef Poedit_welcomescreen_h
 #define Poedit_welcomescreen_h
 
+#include "titleless_window.h"
+
 #include <wx/panel.h>
+#include <wx/frame.h>
+
+#ifdef __WXMSW__
+#include "windows/win10_menubar.h"
+typedef WithWindows10Menubar<TitlelessWindow> WelcomeWindowBase;
+#else
+typedef TitlelessWindow WelcomeWindowBase;
+#endif
+
 
 class PoeditFrame;
 
@@ -36,14 +47,6 @@ protected:
     WelcomeScreenBase(wxWindow *parent);
 };
 
-/// Content view for initially opened Poedit, without a file
-class WelcomeScreenPanel : public WelcomeScreenBase
-{
-public:
-    WelcomeScreenPanel(wxWindow *parent);
-};
-
-
 /// Content view for an empty file (File->New)
 class EmptyPOScreenPanel : public WelcomeScreenBase
 {
@@ -52,4 +55,33 @@ public:
 };
 
 
+/// Window for initially opened Poedit, without a file
+class WelcomeWindow : public WelcomeWindowBase
+{
+public:
+    static WelcomeWindow *GetAndActivate();
+    static WelcomeWindow *GetIfActive()
+        { return ms_instance && ms_instance->IsShown() ? ms_instance : nullptr; }
+
+    /// Hides the welcome window if it is active, and returns true iff it was previously visible
+    static bool HideActive();
+
+    bool ShouldPreventAppExit() const override
+    {
+        return IsShownOnScreen();
+    }
+
+protected:
+    WelcomeWindow();
+    ~WelcomeWindow();
+
+#ifdef __WXMSW__
+    bool ShouldPlaceMenuInNCArea() const override { return false; }
+#endif
+
+private:
+    static WelcomeWindow *ms_instance;
+};
+
+
 #endif // Poedit_welcomescreen_h
